{DeviceAuthenticator} = require "meshblu-authenticator-core"
debug                 = require('debug')('meshblu-authenticator-cwc:device-auth-service')
MeshbluHttp           = require 'meshblu-http'

class DeviceAuthService
  constructor: ({meshbluConfig}) ->
    @meshbluHttp = new MeshbluHttp meshbluConfig
    @meshbluHttp.setPrivateKey new Buffer(meshbluConfig.privateKey, 'base64').toString()

    @authenticatorUuid = meshbluConfig.uuid
    @authenticatorName = meshbluConfig.name || 'meshblu-authenticator-cwc-staging'

    @deviceAuthenticator = new DeviceAuthenticator {@authenticatorUuid, @authenticatorName, @meshbluHttp}

  findAuthorizedCWCUserDevice: (profile, secret,  callback=->) =>
    debug 'findAuthorizedCWCUserDevice'
    {user_id} = profile
    deviceQuery = @getAuthDeviceQuery user_id

    @deviceAuthenticator.findVerified {query: deviceQuery, password: secret}, (error, foundDevice) =>
      debug 'findVerified', error, foundDevice?
      return callback error if error
      return @createCWCUserDevice(profile, secret, callback) unless foundDevice?
      return @meshbluHttp.generateAndStoreToken foundDevice.uuid, callback

  createCWCUserDevice: (profile, secret, callback=->) =>
    debug 'createCWCUserDevice'
    {user_id, displayName, principal} = profile
    [firstName, lastName] = displayName.split(' ')

    deviceData =
      name: displayName
      email: principal
      type: 'octoblu:user'
      octoblu:
        firstName: firstName
        lastName: lastName
        email: principal
        optInEmail: false
        termsAcceptedAt: new Date()
      cwc:
        profile: profile

    return callback new Error "Missing user Id" unless user_id
    authDeviceQuery = @getAuthDeviceQuery user_id
    @deviceAuthenticator.create {
      query: authDeviceQuery
      data: deviceData
      user_id: user_id
      secret: secret
    }, (error, device) =>
      debug '@deviceAuthenticator.create', error, device?
      return callback error if error?
      return @meshbluHttp.generateAndStoreToken device.uuid, callback

  getAuthDeviceQuery: (userId) =>
    return {"#{@authenticatorUuid}.id": userId}

module.exports = DeviceAuthService
